import { mergeObj, MixinPropsFields, MixinStyleFields, ThemeSize, ThemeType } from '../ka-tools/sdk/mixins';

/**
 * transition  动画组件
 * @description
 * @tutorial
 * @property {Boolean}			show			是否展示组件 （默认 false ）
 * @property {String}			mode			使用的动画模式 （默认 'fade' ）
 * @property {String | Number}	duration		动画的执行时间，单位ms （默认 '300' ）
 * @property {String}			timingFunction	使用的动画过渡函数 （默认 'ease-out' ）
 * @property {Object}			customStyle		自定义样式
 * @event {Function} before-enter	进入前触发
 * @event {Function} enter			进入中触发
 * @event {Function} after-enter	进入后触发
 * @event {Function} before-leave	离开前触发
 * @event {Function} leave			离开中触发
 * @event {Function} after-leave	离开后触发
 * @example
 */
export interface KaTransitionProps {
  // 是否展示组件
  show?: boolean;
  // 使用的动画模式
  mode?: string;
  // 动画的执行时间，单位ms
  duration?: number;
  // 使用的动画过渡函数
  timingFunction?: string;
}

export const DefaultTransitionData: KaTransitionProps = {
  show: false,
  mode: 'fade',
  duration: 300,
  timingFunction: 'ease-out',
};

export function useDefaultTransitio(data: KaTransitionProps) {
  mergeObj(DefaultTransitionData, data);
}

/**
 * Text 文本
 * @description 此组件集成了文本类在项目中的常用功能
 * @example <ka-text text="我用十年青春,赴你最后之约"></ka-text>
 */
export interface KaTextProps {
  selectable?: boolean;
  useSelect?: boolean;
  space?: string;
  decode?: boolean;
  text?: string | number;
  color?: string;
  size?: string | number;
  align?: string;
  mode?: string;
  bold?: boolean;
}

export const DefaultTextData: KaTextProps = {
  text: '',
};

export function useDefaultText(data: KaTextProps) {
  mergeObj(DefaultTextData, data);
}

/**
 * icon 图标
 */
export interface KaIconProps {
  // 图标名称
  name?: string;
  // 图标颜色
  color?: string;
  // 图标大小
  size?: string | number;
}

/**
 * button 按钮
 * @description Button 按钮
 *
 */
export interface KaButtonProps {
  // 按住后多久出现点击态，单位毫秒 （默认 0 )
  hoverStartTime?: number;
  // 手指松开后点击态保留时间，单位毫秒 （默认 200 )
  hoverStayTime?: number;
  // 节流，一定时间内只能触发一次 （默认 100 )
  throttleTime?: number;
  // 用于 <form> 组件，点击分别会触发 <form> 组件的 submit/reset 事件
  formType?: string;
  // 开放能力，具体请看uniapp稳定关于button组件部分说明
  openType?: string;
  // 是否禁用
  disabled?: boolean;
  // 按钮文字，之所以通过props传入，是因为slot传入的话 nvue中无法控制文字的样式
  text?: string | KaTextProps;
  // 按钮背景颜色
  bgColor?: string;
  // 按钮背景图片，渐变色属于背景图片
  bgImage?: string;
  // 图标
  icon?: string | KaIconProps;
  // 指定返回用户信息的语言，zh_CN 简体中文，zh_TW 繁体中文，en 英文（默认 en ）
  lang?: string;
  // 按钮大小
  size?: ThemeSize;
}

export const DefaultButtonData: KaButtonProps = {
  hoverStartTime: 0,
  hoverStayTime: 200,
  throttleTime: 100,
};

export function useDefaultButton(data: KaButtonProps) {
  mergeObj(DefaultButtonData, data);
}

export interface KaBadgeProps extends MixinPropsFields<KaBadgeProps> {
  // 是否显示圆点 （默认 false ）
  isDot?: boolean;
  // 显示的内容
  value?: number;
  // 是否显示 （默认 true ）
  show?: boolean;
  // 最大值，超过最大值会显示 '{max}+'  （默认999）
  max?: number;
  // 主题类型，error|warning|success|primary （默认 'error' ）
  type?: ThemeType;
  // 当数值为 0 时，是否展示 Badge （默认 false ）
  showZero?: boolean;
  // 背景颜色，优先级比type高，如设置，type参数会失效
  bgColor?: string;
  // 字体颜色 （默认 '#ffffff' ）
  color?: string;
  // circle-四角均为圆角，horn-左下角为直角（默认 'circle' ）
  shape?: 'circle' | 'horn';
  // 数字的显示方式，overflow|ellipsis|limit  （默认 'overflow' ）
  numberType?: KaBadgeNumberType;
  // 是否绝对定位（默认 false ）
  absolute?: boolean;
  // 设置badge的位置偏移，格式为 [x, y]，也即设置的为top和right的值，absolute为true时有效
  offset?: string[] | number[];
  // 是否反转背景和字体颜色（默认 false ）
  inverted?: boolean;
  // 层级（默认 1 ）
  zIndex?: number;
}
export enum KaBadgeNumberType {
  overflow = 'overflow',
  ellipsis = 'ellipsis',
  limit = 'limit',
}
export const DefaultBadgeData: KaBadgeProps = {
  isDot: false,
  value: 0,
  show: true,
  max: 999,
  type: ThemeType.error,
  showZero: false,
  shape: 'circle',
  numberType: KaBadgeNumberType.overflow,
  offset: [],
  inverted: false,
  absolute: false,
};

export function useDefaultBadge(data: KaBadgeProps) {
  mergeObj(DefaultBadgeData, data);
}

/**
 * Tabs 标签
 * @description tabs标签组件，在标签多的时候，可以配置为左右滑动，标签少的时候，可以禁止滑动。 该组件的一个特点是配置为滚动模式时，激活的tab会自动移动到组件的中间位置。
 */
export interface KaTabsProps {
  duration?: number;
  list?: () => KaTabsItemProps[];
  lineColor?: string;
  inactiveStyle?: () => MixinStyleFields;
  activeStyle?: () => MixinStyleFields;
  lineWidth?: number | string;
  lineHeight?: number | string;
  lineBgSize?: string;
  itemStyle?: () => MixinStyleFields;
  scrollable?: boolean;
  current?: number;
  keyName?: string;
  badgeProps?: () => KaBadgeProps;
}
export interface KaTabsItemProps {
  disabled?: boolean;
  badge?: () => KaBadgeProps;
  rect?: UniApp.NodeInfo;
}

export const DefaultTabsData: KaTabsProps = {
  duration: 300,
  list: () => [],
  lineColor: '#3c9cff',
  activeStyle: () => ({
    color: '#303133',
  }),
  inactiveStyle: () => ({
    color: '#606266',
  }),
  lineWidth: 20,
  lineHeight: 3,
  lineBgSize: 'cover',
  itemStyle: () => ({
    height: '40px',
  }),
  scrollable: true,
  current: 0,
  keyName: 'name',
};

export function useDefaultTabs(data: KaTabsProps) {
  mergeObj(DefaultTabsData, data);
}
