import { KaToastProps } from '../ka-popup/main';
import { mergeObj, MixinPropsFields, MixinStyleFields, ThemeSize, ThemeType } from '../ka-tools/sdk/mixins';

/**
 * ka-status-bar 状态栏占位
 * @description 本组件主要用于状态填充，比如在自定导航栏的时候，它会自动适配一个恰当的状态栏高度。
 */
export interface KaStatusBarProps {
  // 安全区背景色
  bgColor?: string;
}

export let DefaultStatusBarData: KaStatusBarProps = { bgColor: 'transparent' };

export function useDefaultStatusBar(data: KaStatusBarProps) {
  mergeObj(DefaultStatusBarData, data);
}

/**
 * ka-safe-bottom   底部安全区
 * @description 这个适配，主要是针对IPhone X等一些底部带指示条的机型，指示条的操作区域与页面底部存在重合，容易导致用户误操作，因此我们需要针对这些机型进行底部安全区适配。
 */
export interface KaSafeBottomProps {
  // 安全区背景色
  bgColor?: string;
}

export const DefaultSafeBottomData: KaSafeBottomProps = {};

export function useDefaultSafeBottom(data: KaSafeBottomProps) {
  mergeObj(DefaultSafeBottomData, data);
}

/**
 * ka-fixed-bottom  底部定位组件
 */
export interface KaFixedBottomProps {
  // 定位位置，支持左右,全部（左右占满）
  position?: 'left' | 'right' | 'all';
  // 层级
  zIndex?: number;
  // 安全区背景色
  bgColor?: string;
  // 底部距离
  bottom?: number;
}

export const DefaultFixedBottomData: KaFixedBottomProps = {
  position: 'all',
  zIndex: 10,
  bottom: 0,
};

export function useDefaultFixedBottom(data: KaFixedBottomProps) {
  mergeObj(DefaultFixedBottomData, data);
}

/**
 * Navbar 自定义导航栏
 * @description 此组件一般用于在特殊情况下，需要自定义导航栏的时候用到，一般建议使用uni-app带的导航栏。
 */
export interface KaNavbarProps {
  // 是否开启顶部安全区适配  （默认 true ）
  safeTop?: boolean;
  // 导航栏是否固定在顶部 （默认 false ）
  fixed?: boolean;
  // 固定在顶部时，是否生成一个等高元素，以防止塌陷 （默认 false ）
  placeholder?: boolean;
  // 导航栏底部是否显示下边框 （默认 false ）
  border?: boolean;
  // 是否隐藏左侧图标
  hideLeftIcon?: boolean;
  // 左边返回图标的名称，只能为自带的图标 （默认 'arrow-left' ）
  leftIcon?: string;
  // 左边的提示文字
  leftText?: string;
  // 右边的提示文字
  rightText?: string;
  // 右边图标的名称，只能为自带的图标
  rightIcon?: string;
  // 导航栏标题，如设置为空字符，将会隐藏标题占位区域
  title?: string;
  // 导航栏背景设置 （默认 '#ffffff' ）
  bgColor?: string;
  // 导航栏标题的最大宽度，内容超出会以省略号隐藏 （默认 '400rpx' ）
  titleWidth?: string;
  // 导航栏高度(不包括状态栏高度在内，内部自动加上)（默认 '44px' ）
  height?: number;
  // 左侧返回图标的大小（默认 20px ）
  leftIconSize?: number;
  // 左侧返回图标的颜色（默认 #303133 ）
  leftIconColor?: string;
  // 点击左侧区域(返回图标)，是否自动返回上一页（默认 false ）
  autoBack?: boolean;
  // 标题的样式，对象或字符串
  titleStyle?: string;
  // 重定向方法
  redirectToHome?: () => {};
}
export const DefaultNavbarData: KaNavbarProps = {
  safeTop: true,
  fixed: true,
  border: false,
  hideLeftIcon: false,
  leftIcon: 'arrow-left',
  leftText: '',
  rightText: '',
  rightIcon: '',
  title: '',
  bgColor: '#EDF2FA',
  titleWidth: '400rpx',
  height: 44,
  leftIconSize: 20,
  leftIconColor: '#303133',
  autoBack: true,
  titleStyle: '',
};

export function useDefaultNavbar(data: KaNavbarProps) {
  mergeObj(DefaultFixedBottomData, data);
}

/**
 * Tabbar 底部导航栏子项
 */
export interface KaTabbarItemProps {
  // item标签的名称，作为与hk-tabbar的value参数匹配的标识符
  name?: string | number;
  icon?: string;
  // 内置图标或者绝对路径的图片
  inactiveImage?: string;
  activeImage?: string;
  // 右上角的角标提示信息
  badge?: string | number;
  // 是否显示圆点，将会覆盖badge参数
  dot?: boolean;
  // 描述文本
  text?: string;
  // 跳转的路径
  url?: string;
  // 控制徽标的位置，对象或者字符串形式，可以设置top和right属性
  badgeStyle?: string | MixinStyleFields;
}
/**
 * Tabbar 底部导航栏
 * @description 此组件提供了自定义tabbar的能力。
 */
export interface KaTabbarProps {
  // 当前匹配项的name或下标
  active?: number | string;
  // 是否为iPhoneX留出底部安全距离
  safeBottom?: boolean;
  // 是否显示上方边框
  border?: boolean;
  // 元素层级z-index
  zIndex?: number;
  // 选中标签的颜色
  activeColor?: string;
  // 未选中标签的颜色
  inactiveColor?: string;
  // 是否固定在底部
  fixed?: boolean;
  // fixed定位固定在底部时，是否生成一个等高元素防止塌陷
  placeholder?: boolean;
  // 导航栏高度
  height?: number;
  list?: KaTabbarItemProps[];
}
export const DefaultTabbarData: KaTabbarProps = {
  active: 0,
  safeBottom: true,
  border: true,
  zIndex: 1,
  activeColor: '#1989fa',
  inactiveColor: '#7d7e80',
  fixed: true,
  placeholder: true,
  height: 50,
};

export function useDefaultTabbar(data: KaTabbarProps) {
  mergeObj(DefaultTabbarData, data);
}

/**
 * Page 页面容器
 */
export interface KaPageProps {
  navbar?: () => KaNavbarProps;
  tabbar?: () => KaTabbarProps;
  // 背景颜色
  backgroundColor?: string;
  // 背景图片
  backgroundImage?: string;
  // 隐藏导航栏
  hideNavbar?: boolean;
  // 版权字符串
  copyRightText?: string;
  // 标题
  title?: string;
}

export const DefaultPageData: KaPageProps = {};

export function useDefaultPage(data: KaPageProps) {
  mergeObj(DefaultPageData, data);
}

/**
 * 页面组件计算数据
 */
export interface IComputedPageData {
  // 底部安全距离距离
  safeBottomHeight: number;
}
export const computedPageData = {
  safeBottomHeight: 0,
};
export function setComputedPageData(data: IComputedPageData) {
  mergeObj(computedPageData, data);
}

/**
 * 页面默认参数
 */
export interface IDefaultPageData {
  // 顶部导航高度
  navbarHeight?: number;
  // 顶部导航背景
  navbarBgColor?: string;
  // 顶部导航标题央视
  navbarTitleStyle?: string;
  // 底部导航高度
  tabbarHeight?: number;
  // 跳转首页的方法
  redirectToHome?: () => {};
  // 版本数据信息
  copyRightText?: string;
}
export function useDefaultData(data: IDefaultPageData) {
  if (data.navbarHeight) {
    DefaultNavbarData.height = data.navbarHeight;
  }
  if (data.navbarBgColor) {
    DefaultNavbarData.bgColor = data.navbarBgColor;
  }
  if (data.navbarTitleStyle) {
    DefaultNavbarData.titleStyle = data.navbarTitleStyle;
  }

  if (data.tabbarHeight) {
    DefaultTabbarData.height = data.tabbarHeight;
  }

  if (data.redirectToHome) {
    DefaultNavbarData.redirectToHome = data.redirectToHome;
  }
  if (data.copyRightText) {
    DefaultPageData.copyRightText = data.copyRightText;
  }
}

export function useNavbar(title: string): () => KaNavbarProps {
  return () => ({ title });
}

type IUseTabbar = () => KaTabbarProps;
export function useTabbar(data: KaTabbarProps): IUseTabbar {
  return () => data;
}

// #####################################################
/**
 * 定义全局弹框
 * @param data
 */
export enum PopupType {
  toast = 'PageToast',
}
export function showToast(data: KaToastProps | string) {
  if (typeof data === 'string') {
    data = { message: data };
  }
  uni.$emit(PopupType.toast, data);
}
