import * as test from './test';
import { MixinStyleFields } from './mixins';

/**
 * @description 用于获取用户传递值的px值  如果用户传递了"xxpx"或者"xxrpx"，取出其数值部分，如果是"xxxrpx"还需要用过uni.upx2px进行转换
 * @param {number|string} value 用户传递值的px值
 * @param {boolean} unit
 * @returns {number|string}
 */
export function getPx(value: string | number, unit: boolean = false): number | string {
  let finanValue: string = '';
  if (typeof value === 'number') {
    finanValue = value.toString();
  }

  if (test.number(finanValue)) {
    return unit ? `${finanValue}px` : Number(finanValue);
  }
  // 如果带有rpx，先取出其数值部分，再转为px值
  if (/(rpx|upx)$/.test(finanValue)) {
    return unit ? `${uni.upx2px(parseInt(finanValue))}px` : Number(uni.upx2px(parseInt(finanValue)));
  }
  return unit ? `${parseInt(finanValue)}px` : parseInt(finanValue);
}

/**
 * @description 运行期判断平台
 * @returns {string} 返回所在平台(小写)
 * @link 运行期判断平台 https://uniapp.dcloud.io/frame?id=判断平台
 */
export function os(): string {
  return uni.getSystemInfoSync().platform.toLowerCase();
}

/**
 * @description 获取系统信息同步接口
 * @link 获取系统信息同步接口 https://uniapp.dcloud.io/api/system/info?id=getsysteminfosync
 */
export function sys() {
  return uni.getSystemInfoSync();
}

/**
 * 获取节点
 * @param {Object} _this
 * @param {String} selector
 * @param {Boolean} all
 */
export function getRect(_this: any, selector: string, all: boolean = false): Promise<UniApp.NodeInfo> {
  return new Promise<UniApp.NodeInfo>((resolve) => {
    uni
      .createSelectorQuery()
      .in(_this)
      .select(selector)
      .boundingClientRect((rect) => resolve(rect as UniApp.NodeInfo))
      .exec();
  });
}

/**
 * 获取所有节点
 * @param _this
 * @param selector
 * @param all
 * @returns
 */
export function getRectAll(_this: any, selector: string, all: boolean = false) {
  return new Promise<UniApp.NodeInfo[]>((resolve) => {
    uni
      .createSelectorQuery()
      .in(_this)
      .selectAll(selector)
      .boundingClientRect((rect) => resolve(rect as UniApp.NodeInfo[]))
      .exec();
  });
}

/**
 * @description error提示
 * @param {string} err 错误内容
 */
export function error(err: string) {
  // 开发环境才提示，生产环境不会提示
  if (process.env.NODE_ENV === 'development') {
    console.error(`hkUni提示：${err}`);
  }
}

/**
 * @description 取一个区间数
 * @param {Number} min 最小值
 * @param {Number} max 最大值
 */
export function random(min: number, max: number) {
  if (min >= 0 && max > 0 && max >= min) {
    const gab = max - min + 1;
    return Math.floor(Math.random() * gab + min);
  }
  return 0;
}

/**
 * @param {Number} len uuid的长度
 * @param {Boolean} firstU 将返回的首字母置为"u"
 * @param {Nubmer} radix 生成uuid的基数(意味着返回的字符串都是这个基数),2-二进制,8-八进制,10-十进制,16-十六进制
 */
export function guid(len: number = 32, firstU: boolean = true, radix: number | null = null) {
  const chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split('');
  const uuid = [];
  radix = radix || chars.length;

  if (len) {
    // 如果指定uuid长度,只是取随机的字符,0|x为位运算,能去掉x的小数位,返回整数位
    for (let i = 0; i < len; i++) uuid[i] = chars[0 | (Math.random() * radix)];
  } else {
    let r: number;
    // rfc4122标准要求返回的uuid中,某些位为固定的字符
    uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
    uuid[14] = '4';

    for (let i = 0; i < 36; i++) {
      if (!uuid[i]) {
        r = 0 | (Math.random() * 16);
        uuid[i] = chars[i == 19 ? (r & 0x3) | 0x8 : r];
      }
    }
  }
  // 移除第一个字符,并用u替代,因为第一个字符为数值时,该guuid不能用作id或者class
  if (firstU) {
    uuid.shift();
    return `u${uuid.join('')}`;
  }
  return uuid.join('');
}

/**
* @description 获取父组件的参数，因为支付宝小程序不支持provide/inject的写法
	 this.$parent在非H5中，可以准确获取到父组件，但是在H5中，需要多次this.$parent.$parent.xxx
	 这里默认值等于undefined有它的含义，因为最顶层元素(组件)的$parent就是undefined，意味着不传name
	 值(默认为undefined)，就是查找最顶层的$parent
*  @param {string|undefined} name 父组件的参数名
*/
export function $parent(name: string | undefined = undefined, vm: any) {
  let parent = vm.$parent;
  // 通过while历遍，这里主要是为了H5需要多层解析的问题
  while (parent) {
    // 父组件
    if (parent.$options && parent.$options.name !== name) {
      // 如果组件的name不相等，继续上一级寻找
      parent = parent.$parent;
    } else {
      return parent;
    }
  }
  return false;
}

/**
 * @description 去除空格
 * @param String str 需要去除空格的字符串
 * @param String pos both(左右)|left|right|all 默认both
 */
export function trim(str: string, pos: string = 'both') {
  str = String(str);
  if (pos == 'both') {
    return str.replace(/^\s+|\s+$/g, '');
  }
  if (pos == 'left') {
    return str.replace(/^\s*/, '');
  }
  if (pos == 'right') {
    return str.replace(/(\s*$)/g, '');
  }
  if (pos == 'all') {
    return str.replace(/\s+/g, '');
  }
  return str;
}

/**
 * @description 样式转换
 * 对象转字符串，或者字符串转对象
 * @param {object | string} target 需要转换的目标
 * @returns {object|string}
 */
export function addStyle(target?: MixinStyleFields): MixinStyleFields {
  if (target) {
    return deepMerge({}, target);
  }
  return {};
}

/**
 * 字符串样式转为样式对象
 * @param style
 * @returns
 */
export function string2Style(style?: string | MixinStyleFields): MixinStyleFields {
  // 字符串转字符串，对象转对象情形，直接返回
  if (typeof style === 'string') {
    // 字符串转对象
    // 去除字符串样式中的两端空格(中间的空格不能去掉，比如padding: 20px 0如果去掉了就错了)，空格是无用的
    style = trim(style);
    // 根据";"将字符串转为数组形式
    const styleArray = style.split(';');
    const _style: any = {};
    // 历遍数组，拼接成对象
    for (let i = 0; i < styleArray.length; i++) {
      // 'font-size:20px;color:red;'，如此最后字符串有";"的话，会导致styleArray最后一个元素为空字符串，这里需要过滤
      if (styleArray[i]) {
        const item = styleArray[i].split(':');

        // 驼峰转为中划线的形式，否则css内联样式，无法识别驼峰样式属性名
        const key = item[0].replace(/([A-Z])/g, '-$1').toLowerCase();

        _style[key] = trim(item[1]);
      }
    }
    return _style as MixinStyleFields;
  } else {
    return style || {};
  }
}

/**
 * @description 添加单位，如果有rpx，upx，%，px等单位结尾或者值为auto，直接返回，否则加上px单位结尾
 * @param {string|number} value 需要添加单位的值
 * @param {string} unit 添加的单位名 比如px
 */
export function addUnit(value: string | number = 'auto', unit: string = 'px') {
  value = String(value);
  // 用hkView内置验证规则中的number判断是否为数值
  return test.number(value) ? `${value}${unit}` : value;
}

/**
 * @description 深度克隆
 * @param {object} obj 需要深度克隆的对象
 * @returns { * } 克隆后的对象或者原值
 */
export function deepClone<T>(obj: T): T {
  /** 空 */
  if (!obj) return obj;
  /** 数组 */
  if (Array.isArray(obj)) return obj.map((item) => deepClone(item)) as unknown as T;
  /** 日期 */
  if (obj instanceof Date) return new Date(obj) as unknown as T;
  /** 普通对象 */
  if (typeof obj === 'object') {
    return Object.fromEntries(
      Object.entries(obj).map(([k, v]: [string, any]) => {
        return [k, deepClone(v)];
      }),
    ) as unknown as T;
  }
  /** 基本类型 */
  return obj;
}

/**
 * @description JS对象深度合并
 * @param {T} target 需要拷贝的对象
 * @param {T} source 拷贝的来源对象
 * @param {T} others 剩余藏书，依次将后一个参数赋值给前一个
 * @returns {T} 深度合并后的对象
 */
export function deepMerge<T extends object>(target: T, source: T, ...others: T[]): T {
  function doMerge(t: T, s: T) {
    const result: any = { ...t };
    for (const key in s) {
      if (Object.prototype.hasOwnProperty.call(s, key)) {
        const value = s[key];

        if (test.object(value) && !Array.isArray(value) && value !== null) {
          result[key] = doMerge(t[key] as any, value as any);
        } else {
          if (!test.empty(value)) {
            result[key] = value;
          }
        }
      }
    }
    return result;
  }

  let result = doMerge(target, source);
  // console.log(others);
  if (others.length > 0)
    others.forEach((o) => {
      // console.log(o);
      if (o) result = doMerge(result, o);
    });

  return result;
}

/**
 * 文字脱敏规则
 * @param {string} value
 * @return
 */
export function encryptText(value: string): string {
  let finalValue = '';
  if (value.length === 2) {
    finalValue = value.substring(0, 1) + '*';
  } else if (value.length > 2) {
    finalValue = value.substring(0, 1) + '***' + value.substring(value.length - 1, value.length);
  } else {
    finalValue = value;
  }
  return finalValue;
}

/**
 * @param {number} time 等待时间毫秒数
 * @description 睡眠
 */
export function sleep(time: number = 200) {
  return new Promise((resolve) => {
    setTimeout(resolve, time);
  });
}

// 定义一个一定时间后自动成功的promise，让调用nextTick方法处，进入下一个then方法
export const nextTick = () => new Promise((resolve) => setTimeout(resolve, 1000 / 50));
